"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

exports.default = function (_ref) {
    var t = _ref.types;

    function getRuntimeModuleName(opts) {
        return opts.moduleName || "babel-runtime";
    }

    function has(obj, key) {
        return Object.prototype.hasOwnProperty.call(obj, key);
    }

    var HELPER_BLACKLIST = ["interopRequireWildcard", "interopRequireDefault"];

    return {
        pre: function pre(file) {
            var moduleName = getRuntimeModuleName(this.opts);

            if (this.opts.helpers !== false) {
                file.set("helperGenerator", function (name) {
                    if (HELPER_BLACKLIST.indexOf(name) < 0) {
                        var tmp = file.addImport(moduleName + "/helpers/" + name, "default", name); // 降低scope uid
                        tmp.name = tmp.name.replace('2', '');
                        return tmp;
                    }
                });
            }

            this.setDynamic("regeneratorIdentifier", function () {
                return file.addImport(moduleName + "/regenerator", "default", "regeneratorRuntime");
            });
        },


        visitor: {
            // 对于函数声明，在helper中的，均进行删除，并且添加helper
            FunctionDeclaration: function FunctionDeclaration(path, state) {
                var funcName = path.node.id.name.replace('_', '').replace('2', '').replace('3', '');
                var isHelper = helperNames.indexOf(funcName) !== -1 && HELPER_BLACKLIST.indexOf(funcName) < 0;
                var isPureHelper = path.node.id.name.indexOf('_') !== -1;
                if (funcName && isHelper && isPureHelper) {
                    for (var i in this.file.dynamicImportIds) {
                        this.file.dynamicImportIds[i].name = this.file.dynamicImportIds[i].name.replace('2', '').replace('3', '');
                    }
                    this.addHelper(funcName);
                    path.remove();
                }
            },

            // 保持原有的插件功能
            ReferencedIdentifier: function ReferencedIdentifier(path, state) {
                var node = path.node,
                    parent = path.parent,
                    scope = path.scope;


                if (node.name === "regeneratorRuntime" && state.opts.regenerator !== false) {
                    path.replaceWith(state.get("regeneratorIdentifier"));
                    return;
                }

                if (state.opts.polyfill === false) return;

                if (t.isMemberExpression(parent)) return;
                if (!has(_definitions2.default.builtins, node.name)) return;
                if (scope.getBindingIdentifier(node.name)) return;

                // Symbol() -> _core.Symbol(); new Promise -> new _core.Promise
                var moduleName = getRuntimeModuleName(state.opts);
                path.replaceWith(state.addImport(moduleName + "/core-js/" + _definitions2.default.builtins[node.name], "default", node.name));
            },


            // arr[Symbol.iterator]() -> _core.$for.getIterator(arr)
            CallExpression: function CallExpression(path, state) {
                if (state.opts.polyfill === false) return;

                // we can't compile this
                if (path.node.arguments.length) return;

                var callee = path.node.callee;
                if (!t.isMemberExpression(callee)) return;
                if (!callee.computed) return;
                if (!path.get("callee.property").matchesPattern("Symbol.iterator")) return;

                var moduleName = getRuntimeModuleName(state.opts);
                path.replaceWith(t.callExpression(state.addImport(moduleName + "/core-js/get-iterator", "default", "getIterator"), [callee.object]));
            },


            // Symbol.iterator in arr -> core.$for.isIterable(arr)
            BinaryExpression: function BinaryExpression(path, state) {
                if (state.opts.polyfill === false) return;

                if (path.node.operator !== "in") return;
                if (!path.get("left").matchesPattern("Symbol.iterator")) return;

                var moduleName = getRuntimeModuleName(state.opts);
                path.replaceWith(t.callExpression(state.addImport(moduleName + "/core-js/is-iterable", "default", "isIterable"), [path.node.right]));
            },


            // Array.from -> _core.Array.from
            MemberExpression: {
                enter: function enter(path, state) {
                    if (state.opts.polyfill === false) return;
                    if (!path.isReferenced()) return;

                    var node = path.node;

                    var obj = node.object;
                    var prop = node.property;

                    if (!t.isReferenced(obj, node)) return;
                    if (node.computed) return;
                    if (!has(_definitions2.default.methods, obj.name)) return;

                    var methods = _definitions2.default.methods[obj.name];
                    if (!has(methods, prop.name)) return;

                    // doesn't reference the global
                    if (path.scope.getBindingIdentifier(obj.name)) return;

                    // special case Object.defineProperty to not use core-js when using string keys
                    if (obj.name === "Object" && prop.name === "defineProperty" && path.parentPath.isCallExpression()) {
                        var call = path.parentPath.node;
                        if (call.arguments.length === 3 && t.isLiteral(call.arguments[1])) return;
                    }

                    var moduleName = getRuntimeModuleName(state.opts);
                    path.replaceWith(state.addImport(moduleName + "/core-js/" + methods[prop.name], "default", obj.name + "$" + prop.name));
                },
                exit: function exit(path, state) {
                    if (state.opts.polyfill === false) return;
                    if (!path.isReferenced()) return;

                    var node = path.node;

                    var obj = node.object;

                    if (!has(_definitions2.default.builtins, obj.name)) return;
                    if (path.scope.getBindingIdentifier(obj.name)) return;

                    var moduleName = getRuntimeModuleName(state.opts);
                    path.replaceWith(t.memberExpression(state.addImport(moduleName + "/core-js/" + _definitions2.default.builtins[obj.name], "default", obj.name), node.property, node.computed));
                }
            }
        }
    };
};

var _definitions = require("./definitions");

var _definitions2 = _interopRequireDefault(_definitions);

var _helpers = require("babel-helpers/lib/helpers");

var _helpers2 = _interopRequireDefault(_helpers);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var helperNames = Object.keys(_helpers2.default);

module.exports = exports["default"];